//////////////////////////////////////////////////////////////////////////////
/*!
  \file		EpcsTwain.h

  \brief	Custom definitions of EPSON Scan for Business.

  \author       SEIKO EPSON CORPORATION

  \date
  - 2009/07/04  Create
  */
//////////////////////////////////////////////////////////////////////////////
#ifndef	_EPCSTWAIN_H_
#define	_EPCSTWAIN_H_

#include "twain.h"
#define	EPCS_CO_VERSION						0x0001

/****************************************************************************
 * Condition code                                                           *
 ****************************************************************************/
#define TWCC_EPCS_CUSTOMBASE				0xA000
#define TWCC_EPCS_NETWORKUNREACHABLE		(TWCC_EPCS_CUSTOMBASE + 0x0001)
#define TWCC_EPCS_PAPERTIMEOUT				(TWCC_EPCS_CUSTOMBASE + 0x0002)

/****************************************************************************
 * Data Argument                                                            *
 ****************************************************************************/


/****************************************************************************
 * Message                                                                  *
 ****************************************************************************/


/****************************************************************************
 * Capabilities                                                             *
 ****************************************************************************/
#define CAP_EPCS_CUSTOMBASE					0xA000
#define	ICAP_EPCS_TEXTENHANCEMENT			(CAP_EPCS_CUSTOMBASE + 0x0001)
#define	ICAP_EPCS_BLANKPAGESKIP				(CAP_EPCS_CUSTOMBASE + 0x0002)
#define	ICAP_EPCS_COLORENHANCEMENT			(CAP_EPCS_CUSTOMBASE + 0x0003)
#define	CAP_EPCS_PRINTERPOSITIONX			(CAP_EPCS_CUSTOMBASE + 0x0004)
#define	CAP_EPCS_PRINTERPOSITIONY			(CAP_EPCS_CUSTOMBASE + 0x0005)
#define	CAP_EPCS_PRINTDIRECTION				(CAP_EPCS_CUSTOMBASE + 0x0006)
#define	CAP_EPCS_PRINTFONTSIZE				(CAP_EPCS_CUSTOMBASE + 0x0007)
#define	ICAP_EPCS_ADAPTIVETHRESHOLD			(CAP_EPCS_CUSTOMBASE + 0x0008)
#define	ICAP_EPCS_MICRFONT					(CAP_EPCS_CUSTOMBASE + 0x0009)
#define	ICAP_EPCS_MICRMETHOD				(CAP_EPCS_CUSTOMBASE + 0x000A)
#define	ICAP_EPCS_OCRABENABLED				(CAP_EPCS_CUSTOMBASE + 0x000B)
#define	ICAP_EPCS_OCRABDIRECTION			(CAP_EPCS_CUSTOMBASE + 0x000C)
#define	ICAP_EPCS_OCRABFONT					(CAP_EPCS_CUSTOMBASE + 0x000D)
#define	ICAP_EPCS_OCRABFRAME				(CAP_EPCS_CUSTOMBASE + 0x000E)
#define	ICAP_EPCS_OCRABSIDE					(CAP_EPCS_CUSTOMBASE + 0x000F)
#define	ICAP_EPCS_OCRABSPACEHANDLING		(CAP_EPCS_CUSTOMBASE + 0x0010)
#define	CAP_EPCS_EJECTDIRECTION				(CAP_EPCS_CUSTOMBASE + 0x0011)
#define	CAP_EPCS_SCANSIDE					(CAP_EPCS_CUSTOMBASE + 0x0012)
#define	CAP_EPCS_SCANDIRECTION				(CAP_EPCS_CUSTOMBASE + 0x0013)
#define	CAP_EPCS_DUPLEXORDER				(CAP_EPCS_CUSTOMBASE + 0x0014)
#define CAP_EPCS_AUTOEJECT					(CAP_EPCS_CUSTOMBASE + 0x0015)

/****************************************************************************
 * Capability Constants                                                     *
 ****************************************************************************/

/* ICAP_EPCS_BLANKPAGESKIP values (BS_means Blankpage Skip) */
#define	TWEPCS_BS_OFF						0
#define	TWEPCS_BS_LOW						1
#define	TWEPCS_BS_MEDIUM					2
#define	TWEPCS_BS_HIGH						3

/* ICAP_EPCS_COLORENHANCEMENT values (CE_means Color Enhancement) */
#define	TWEPCS_CE_NONE						0
#define	TWEPCS_CE_RED						1
#define	TWEPCS_CE_GREEN						2
#define	TWEPCS_CE_BLUE						3

/* CAP_PRINTER values (PR_ means printer) */
#define	TWPR_EPCS_CUSTOMPRINTER				0xA000
#define	TWPR_EPCS_ENDORSERTOPELECTRIC		(TWPR_EPCS_CUSTOMPRINTER + 0x0001)
#define	TWPR_EPCS_ENDORSERBOTTOMELECTRIC	(TWPR_EPCS_CUSTOMPRINTER + 0x0002)

/* CAP_EPCS_PRINTDIRECTION values (PD_means Print Direction) */
#define	TWEPCS_PD_LEFTRIGHT					0
#define	TWEPCS_PD_RIGHTLEFT					1
#define	TWEPCS_PD_TOPBOTTOM					2
#define	TWEPCS_PD_BOTTOMTOP					3

/* ICAP_EPCS_MICRFONT values (MF_means MICR Font) */
#define	TWEPCS_MF_E13B						0
#define	TWEPCS_MF_CMC7						1
#define	TWEPCS_MF_CMC7_ALPHANUM				2
#define	TWEPCS_MF_CMC7_NUM					TWEPCS_MF_CMC7

/* ICAP_EPCS_MICRMETHOD values (MM_means MICR Method) */
#define	TWEPCS_MM_MICRONLY					0
#define	TWEPCS_MM_OCRONLY					1
#define	TWEPCS_MM_MICRANDOCR				2

/* ICAP_EPCS_OCRABDIRECTION values (OD_means OCR Direction) */
#define	TWEPCS_OD_LEFTRIGHT					0
#define	TWEPCS_OD_TOPBOTTOM					1
#define	TWEPCS_OD_RIGHTLEFT					2
#define	TWEPCS_OD_BOTTOMTOP					3

/* ICAP_EPCS_OCRABFONT values (OF_means OCR Font) */
#define	TWEPCS_OF_OCR_FONT_OCRA_NUM					0
#define	TWEPCS_OF_OCR_FONT_OCRA_ALPHA				1
#define	TWEPCS_OF_OCR_FONT_OCRA_ALPHANUM			2
#define	TWEPCS_OF_OCR_FONT_OCRA_ALPHANUM_WOOH		3
#define	TWEPCS_OF_OCR_FONT_OCRA_ALPHANUM_WOZERO		4
#define TWEPCS_OF_OCR_FONT_OCRA_SYMNUM				5
#define	TWEPCS_OF_OCR_FONT_OCRB_NUM					6
#define	TWEPCS_OF_OCR_FONT_OCRB_ALPHA				7
#define	TWEPCS_OF_OCR_FONT_OCRB_ALPHANUM			8
#define	TWEPCS_OF_OCR_FONT_OCRB_ALPHANUM_WOOH		9
#define	TWEPCS_OF_OCR_FONT_OCRB_ALPHANUM_WOZERO		10
#define TWEPCS_OF_OCR_FONT_OCRB_SYMNUM				11

/* ICAP_EPCS_OCRABSIDE values (OS_means OCR_AB Side) */
#define	TWEPCS_OS_FRONT						0
#define	TWEPCS_OS_BACK						1

/* ICAP_EPCS_OCRABSPACEHANDLING values (SH_means OCR_AB Space Handling) */
#define	TWEPCS_SH_SPACEENABLE				0
#define	TWEPCS_SH_SPACEDISABLE				1

/* CAP_EPCS_EJECTDIRECTION values (ED_means Eject Direction) */
#define	TWEPCS_ED_FRONT						0
#define	TWEPCS_ED_REAR						1

/* CAP_EPCS_SCANSIDE values (SS_means Scan Side) */
#define	TWEPCS_SS_FRONT						0
#define	TWEPCS_SS_BACK						1

/* CAP_EPCS_SCANDIRECTION values (SD_means Scan Direction) */
#define	TWEPCS_SD_FRONTTOREAR				0
#define	TWEPCS_SD_REARTOFRONT				1

/* ICAP_EPCS_DUPLEXORDER values (DO_means Duplex Order) */
#define TWEPCS_DO_LOWERFIRST				0
#define TWEPCS_DO_UPPERFIRST				1

/* ICAP_COMPRESSION values (CP_means ComPression) */
#define	TWCP_EPCS_CUSTOMCOMP				0xA000
#define	TWCP_EPCS_JPEGHIGH					(TWCP_EPCS_CUSTOMCOMP + 0x0001)
#define	TWCP_EPCS_JPEGNORMAL				(TWCP_EPCS_CUSTOMCOMP + 0x0002)
#define	TWCP_EPCS_JPEGLOW					(TWCP_EPCS_CUSTOMCOMP + 0x0003)
#define	TWCP_EPCS_PDFHIGH					(TWCP_EPCS_CUSTOMCOMP + 0x0004)
#define	TWCP_EPCS_PDFNORMAL					(TWCP_EPCS_CUSTOMCOMP + 0x0005)


/****************************************************************************
 * Extended Image Info Attributes                                           *
 ****************************************************************************/
#define	TWEI_EPCS_CUSTOMEXT					0xA000
#define	TWEI_EPCS_OCRABDATA					(TWEI_EPCS_CUSTOMEXT + 0x0001)


/****************************************************************************
 * Structure Definitions                                                    *
 ****************************************************************************/


#endif // _EPCSTWAIN_H_
