
Imports EpsonPLQDriver

Public Class MainForm

    Protected m_objApi As APIControl = Nothing

    Protected Const DIALOG_CAPTION As String = "PLQ-22 Sample Program"

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.ComboBoxFormat.SelectedIndex = 0
        Me.GroupBoxIBMDataType.Enabled = True
        EnableButton(True)

        m_objApi = New APIControl()
        m_objApi.MainFormItem(Me)

        ' Create PLQ-22 API interface
        If Not m_objApi.InitInstance() Then
            ' If failed, the program will terminate
            Me.Close()
        Else
            ' Retrieve and display device status information
            GetDeviceStatus()
        End If
    End Sub

    ' Actions taken after MICR Read button pressed
    Private Sub ButtonMicrRead_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonMicrRead.Click
        Dim eMfMicrType As MfMicrType = MfMicrType.MF_MICR_USE_MICR_RETRY
        Dim eMfMicrFont As MfMicrFont = MfMicrFont.MF_MICR_FONT_E13B
        Dim bSave As Boolean = False
        Dim strMicr As String = ""

        Me.TextBoxMicr.Text = ""
        Me.TextBoxMicr.Refresh()
        EnableButton(False)

        If Me.RadioButtonMicrOcr.Checked Then
            eMfMicrType = MfMicrType.MF_MICR_USE_MICOCR_RETRY2
        End If

        If Me.RadioButtonCMC7.Checked Then
            eMfMicrFont = MfMicrFont.MF_MICR_FONT_CMC7
        End If

        bSave = Me.CheckBoxSaveMicr.Checked

        strMicr = m_objApi.ReadMICR(eMfMicrType, eMfMicrFont, bSave)

        Me.TextBoxMicr.Text = strMicr

        EnableButton(True)
    End Sub

    ' Actions taken after Format combo box selection changed
    Private Sub ComboBoxFormat_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBoxFormat.SelectedIndexChanged
        Dim eMsrwFormat As MsrwFormat
        eMsrwFormat = GetFormat()
        If eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604 OrElse eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746 Then
            Me.GroupBoxIBMDataType.Enabled = True
        Else
            Me.GroupBoxIBMDataType.Enabled = False
        End If
    End Sub

    ' Actions taken after MS Read button pressed
    Private Sub ButtonMsRead_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonMsRead.Click
        Dim eMsrwFormat As MsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
        Dim eMsrwIBMDataTyoe As MsrwIBMDataType = MsrwIBMDataType.EPS_BI_MSRW_EOM_NONE
        Dim bSave As Boolean = False
        Dim strMsr As String = ""

        Me.TextBoxRead.Text = ""
        Me.TextBoxRead.Refresh()
        EnableButton(False)

        eMsrwFormat = GetFormat()

        If eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604 OrElse eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746 Then
            If Me.RadioButton3C.Checked Then
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3C
            Else
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3F
            End If
        End If

        bSave = Me.CheckBoxSaveMsr.Checked

        strMsr = m_objApi.ReadStripe(eMsrwFormat, eMsrwIBMDataTyoe, bSave)

        Me.TextBoxRead.Text = strMsr

        EnableButton(True)
    End Sub

    ' Actions taken after MS Write button pressed
    Private Sub ButtonMsWrite_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonMsWrite.Click
        Dim eMsrwFormat As MsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
        Dim eMsrwIBMDataTyoe As MsrwIBMDataType = MsrwIBMDataType.EPS_BI_MSRW_EOM_NONE
        Dim strMsw As String = ""

        If MessageBox.Show("Click OK to write to magnetic stripe?", DIALOG_CAPTION, MessageBoxButtons.OKCancel, _
                            MessageBoxIcon.Question) = Windows.Forms.DialogResult.Cancel Then
            Exit Sub
        End If

        strMsw = Me.TextBoxWrite.Text

        If CheckWriteString(strMsw) Then
            Exit Sub
        End If

        EnableButton(False)

        eMsrwFormat = GetFormat()

        If eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604 OrElse eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746 Then
            If Me.RadioButton3C.Checked Then
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3C
            Else
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3F
            End If
        End If

        m_objApi.WriteStripe(eMsrwFormat, eMsrwIBMDataTyoe, strMsw)

        EnableButton(True)
    End Sub

    ' Actions taken after MS Clear button pressed
    Private Sub ButtonMsClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonMsClear.Click
        Dim eMsrwFormat As MsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
        Dim eMsrwIBMDataTyoe As MsrwIBMDataType = MsrwIBMDataType.EPS_BI_MSRW_EOM_NONE

        If MessageBox.Show("Click OK to erase magnetic stripe?", DIALOG_CAPTION, MessageBoxButtons.OKCancel, _
                            MessageBoxIcon.Question) = Windows.Forms.DialogResult.Cancel Then
            Exit Sub
        End If

        EnableButton(False)

        eMsrwFormat = GetFormat()

        If eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604 OrElse eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746 Then
            If Me.RadioButton3C.Checked Then
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3C
            Else
                eMsrwIBMDataTyoe = MsrwIBMDataType.EPS_BI_MSRW_EOM_3F
            End If
        End If

        m_objApi.ClearStripe(eMsrwFormat, eMsrwIBMDataTyoe)

        EnableButton(True)
    End Sub

    ' Actions taken after Exit button pressed
    Private Sub ButtonExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonExit.Click
        m_objApi.Close()
        Me.Close()
    End Sub

    Delegate Sub SetDeviceStatusDelegate(ByVal eEscMcASBType As EscMcASBType, ByVal eEscMcASB As EscMcASB)

    Public Sub SetDeviceStatusCallback(ByVal eEscMcASBType As EscMcASBType, ByVal eEscMcASB As EscMcASB)
        If Me.InvokeRequired Then
            Dim c As SetDeviceStatusDelegate = New SetDeviceStatusDelegate(AddressOf SetDeviceStatus)
            Me.Invoke(c, New Object() {eEscMcASBType, eEscMcASB})
        Else
            Me.SetDeviceStatus(eEscMcASBType, eEscMcASB)
        End If
    End Sub

    ' Populate check boxes in Status area
    Public Sub SetDeviceStatus(ByVal eEscMcASBType As EscMcASBType, ByVal eEscMcASB As EscMcASB)
        If eEscMcASBType = EscMcASBType.EPS_STATUS_INDEX_GENERAL AndAlso eEscMcASB = EscMcASB.ESCMC_ASB_NO_RESPONSE Then
            Me.CheckBoxNoResponse.Checked = True
            Me.CheckBoxOffline.Checked = False
            Me.CheckBoxPaperFeed.Checked = False
            Me.CheckBoxPaperSet.Checked = False
            Me.CheckBoxCoverStatus.Checked = False

            Me.CheckBoxNoPaper.Checked = False
            Me.CheckBoxPaperEject.Checked = False
            Me.CheckBoxCoverOpen.Checked = False

            Me.CheckBoxFatalError.Checked = False

            Me.CheckBoxHeadHotError.Checked = False
        Else
            Me.CheckBoxNoResponse.Checked = False
            Select Case eEscMcASBType
                Case EscMcASBType.EPS_STATUS_INDEX_GENERAL
                    Me.CheckBoxOffline.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_OFFLINE)
                    Me.CheckBoxPaperFeed.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_PAPER_FEED)
                    Me.CheckBoxPaperSet.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_PAPER_SET)
                    Me.CheckBoxCoverStatus.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_COVER_STATUS)
                Case EscMcASBType.EPS_STATUS_INDEX_RETURNNABLE_ERROR
                    Me.CheckBoxNoPaper.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_NO_PAPER)
                    Me.CheckBoxPaperEject.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_PAPER_EJECT)
                    Me.CheckBoxCoverOpen.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_COVER_OPEN)
                Case EscMcASBType.EPS_STATUS_INDEX_UNRETURNABLE_ERROR
                    Me.CheckBoxFatalError.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_FATAL_ERROR)
                Case EscMcASBType.EPS_STATUS_INDEX_AUTO_CARRIAGE_RETURN_ERROR
                    Me.CheckBoxHeadHotError.Checked = CBool(eEscMcASB And EscMcASB.ESCMC_ASB_HEAD_HOT_ERROR)
            End Select
        End If
    End Sub

    ' Populate check boxes in Status area
    Private Sub GetDeviceStatus()
        Dim eEscMcASB As EscMcASB() = Nothing
        Dim eEscMcASBType As EscMcASBType = EscMcASBType.EPS_STATUS_INDEX_GENERAL
        Dim nCount As Integer = 0

        eEscMcASB = m_objApi.GetStatus()
        For nCount = 0 To eEscMcASB.Length - 1 Step 1
            Select Case nCount
                Case 0
                    eEscMcASBType = EscMcASBType.EPS_STATUS_INDEX_GENERAL
                Case 1
                    eEscMcASBType = EscMcASBType.EPS_STATUS_INDEX_RETURNNABLE_ERROR
                Case 2
                    eEscMcASBType = EscMcASBType.EPS_STATUS_INDEX_UNRETURNABLE_ERROR
                Case 3
                    eEscMcASBType = EscMcASBType.EPS_STATUS_INDEX_AUTO_CARRIAGE_RETURN_ERROR
                Case Else
            End Select
            SetDeviceStatus(eEscMcASBType, eEscMcASB(nCount))
        Next
    End Sub

    ' Enable/Disable buttons
    Private Sub EnableButton(ByVal bEnable As Boolean)
        Me.ButtonMicrRead.Enabled = bEnable
        Me.ButtonMsRead.Enabled = bEnable
        Me.ButtonMsWrite.Enabled = bEnable
        Me.ButtonMsClear.Enabled = bEnable
    End Sub

    ' Retrieve a value from the Format combo box
    Private Function GetFormat() As MsrwFormat
        Dim eMsrwFormat As MsrwFormat

        Select Case (Me.ComboBoxFormat.SelectedIndex)
            Case 0
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
            Case 1
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_DINISO
            Case 2
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_ANSI
            Case 3
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM4746
            Case 4
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_ISO7811
            Case 5
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_HT2751CIZ
            Case 6
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_ISO8484
            Case Else
                eMsrwFormat = MsrwFormat.EPS_BI_MSRW_FORMAT_IBM3604
        End Select

        Return eMsrwFormat
    End Function

    ' Validate data entry in Write text box
    Private Function CheckWriteString(ByVal strWriteString As String) As Boolean
        Dim cWriteString As Char

        ' Check if field is blank
        If strWriteString.Equals("") Then
            MessageBox.Show("Text specified to be written to magnetic stripe is empty.", _
                                DIALOG_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Warning)
            Return True
        End If

        ' Check if entered characters are valid
        For Each cWriteString In strWriteString.ToCharArray()
            If Asc(cWriteString) < &H30 OrElse Asc(cWriteString) > &H3F Then
                MessageBox.Show("Text specified to be written to magnetic stripe is invalid.", _
                                DIALOG_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Return True
            End If
        Next

        ' Check if the number of characters does not exceed the limitation
        If strWriteString.Length > 105 Then
            MessageBox.Show("Too many characters specified for text to write to magnetic stripe." & vbCrLf & _
                                "Text length must be less than or equal to 105 characters.", _
                                DIALOG_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Warning)
            Return True
        End If

        Return False
    End Function

End Class
