// APIControl.cpp : implementation file
//

#include "stdafx.h"
#include "SampleProgram.h"
#include "SampleProgramDlg.h"
#include "APIControl.h"


#define PAPER_INSERT_TIMEOUT	10
#define PASSBOOK_INSERT_TIMEOUT	10000
#define PAPER_EJECT_TIMEOUT		5000

// This macro checks the return value of PLQ-22 API.
// If an error is returned, an error message is displayed and the error flag is set to true.
#define CheckResponse(a) {\
	int ret = (a);\
	m_ErrFlag = false;\
	if(ret != SUCCESS){\
		::MessageBox((AfxGetApp()->m_pMainWnd)->GetSafeHwnd(), #a, GetResultString(ret), MB_OK | MB_ICONERROR);\
		m_ErrFlag = true;\
	}\
}


// Callback function called when device status notification occurs
int CALLBACK cbMultiStatus(EPBS_UINT32 dwStatusType, EPBS_UINT32 dwStatus, LPSTR)
{
	((CSampleProgramDlg*)AfxGetApp()->m_pMainWnd)->SetDeviceStatus(dwStatusType, dwStatus);
	return 0;
}


// CAPIControl

IMPLEMENT_DYNAMIC(CAPIControl, CWnd)

CAPIControl::CAPIControl(void)
{
	m_hLibrary = NULL;
	m_nHandle = 0;
	m_ErrFlag = false;
}

CAPIControl::~CAPIControl(void)
{
	if(m_hLibrary != NULL)
	{
		Exit();
		::FreeLibrary(m_hLibrary);
	}
}


BEGIN_MESSAGE_MAP(CAPIControl, CWnd)
END_MESSAGE_MAP()



// CAPIControl message handlers


bool CAPIControl::InitInstance(void)
{
	m_hLibrary = ::LoadLibrary("PLQStatusApi.dll");
	if(m_hLibrary == NULL)
	{
		MessageBox(_T("Failure LoadLibrary"), DIALOG_CAPTION, MB_OK | MB_ICONERROR);
		return false;
	}

	BiOpenMonPrinter				= (DLL_BiOpenMonPrinter)				GetProcAddress(m_hLibrary, "BiOpenMonPrinter");
	BiGetStatus						= (DLL_BiGetStatus)						GetProcAddress(m_hLibrary, "BiGetStatus");
	BiGetMultiStatus				= (DLL_BiGetMultiStatus)				GetProcAddress(m_hLibrary, "BiGetMultiStatus");
	BiSetStatusBackFunction			= (DLL_BiSetStatusBackFunction)			GetProcAddress(m_hLibrary, "BiSetStatusBackFunction");
	BiSetStatusBackFunctionEx		= (DLL_BiSetStatusBackFunctionEx)		GetProcAddress(m_hLibrary, "BiSetStatusBackFunctionEx");
	BiSetMultiStatusBackFunction	= (DLL_BiSetMultiStatusBackFunction)	GetProcAddress(m_hLibrary, "BiSetMultiStatusBackFunction");
	BiCancelStatusBack				= (DLL_BiCancelStatusBack)				GetProcAddress(m_hLibrary, "BiCancelStatusBack");
	BiResetPrinter					= (DLL_BiResetPrinter)					GetProcAddress(m_hLibrary, "BiResetPrinter");
	BiSCNMICRFunction				= (DLL_BiSCNMICRFunction)				GetProcAddress(m_hLibrary, "BiSCNMICRFunction");
	BiGetPrnCapability				= (DLL_BiGetPrnCapability)				GetProcAddress(m_hLibrary, "BiGetPrnCapability");
	BiCloseMonPrinter				= (DLL_BiCloseMonPrinter)				GetProcAddress(m_hLibrary, "BiCloseMonPrinter");
	BiGetVersion					= (DLL_BiGetVersion)					GetProcAddress(m_hLibrary, "BiGetVersion");
	BiMICRClearSpaces				= (DLL_BiMICRClearSpaces)				GetProcAddress(m_hLibrary, "BiMICRClearSpaces");
	BiMSRWSetting					= (DLL_BiMSRWSetting)					GetProcAddress(m_hLibrary, "BiMSRWSetting");
	BiMSRWReadStripeData			= (DLL_BiMSRWReadStripeData)			GetProcAddress(m_hLibrary, "BiMSRWReadStripeData");
	BiMSRWGetStripeData				= (DLL_BiMSRWGetStripeData)				GetProcAddress(m_hLibrary, "BiMSRWGetStripeData");
	BiMSRWWriteStripeData			= (DLL_BiMSRWWriteStripeData)			GetProcAddress(m_hLibrary, "BiMSRWWriteStripeData");
	BiMSRWClearStripeData			= (DLL_BiMSRWClearStripeData)			GetProcAddress(m_hLibrary, "BiMSRWClearStripeData");
	BiEjectPaper					= (DLL_BiEjectPaper)					GetProcAddress(m_hLibrary, "BiEjectPaper");
	BiLockPort						= (DLL_BiLockPort)						GetProcAddress(m_hLibrary, "BiLockPort");
	BiUnlockPort					= (DLL_BiUnlockPort)					GetProcAddress(m_hLibrary, "BiUnlockPort");


	// Connect to device
	// If connection failed, this program displays a message and tries to reconnect.
	do
	{
		m_nHandle = BiOpenMonPrinter(TYPE_PRINTER, "PLQ-22U");
		if(m_nHandle <= 0)
		{
			if(MessageBox(_T("Unable to connect to printer\nEnsure Driver installed correctly\nand Printer is powered on"),
				GetResultString(m_nHandle), MB_OKCANCEL | MB_ICONERROR) == IDCANCEL)
			{
				return false;
			}
			::Sleep(1000);
		}
	} while (m_nHandle <= 0);


	// Register callback function for device status notification
	CheckResponse(BiSetMultiStatusBackFunction(m_nHandle, cbMultiStatus));
	if(m_ErrFlag)
	{
		return false;
	}

	return true;
}

void CAPIControl::Exit(void)
{
	if(m_nHandle > 0)
	{
		// Cancel callback function for device status notification
		CheckResponse(BiCancelStatusBack(m_nHandle));
		// Disconnect from device
		CheckResponse(BiCloseMonPrinter(m_nHandle));
		m_nHandle = 0;
	}
}

// Read MICR
CString CAPIControl::ReadMICR(EPBS_UINT8 bSelect, EPBS_UINT8 bFont, bool bSave)
{
	int nResult = SUCCESS;
	CString strMICR;
	strMICR.Empty();

	// Lock the port
	CheckResponse(BiLockPort(m_nHandle, 0));
	if(m_ErrFlag)
	{
		return strMICR;
	}

	// Set MICR settings in driver
	ConfigureMICR(bSelect, bFont);
	if(m_ErrFlag)
	{
		// Unlock the port
		CheckResponse(BiUnlockPort(m_nHandle));
		return strMICR;
	}

	// Process MICR
	nResult = BiSCNMICRFunction(m_nHandle, NULL, MF_EXEC);
	strMICR = m_tMicr.szMicrStr;
	if(nResult != SUCCESS)
	{
		strMICR += _T(" ( ");
		strMICR += GetResultString(nResult);
		strMICR += _T(" ) ");
	}
	else
	{
		if(m_tMicr.iRet != SUCCESS)
		{
			strMICR += _T(" ( ");
			strMICR += GetResultString(m_tMicr.iRet);
			strMICR += _T(" ) ");
		}
	}

	// Unlock the port
	CheckResponse(BiUnlockPort(m_nHandle));

	if(bSave)
	{
		SaveMICR(nResult);
	}

	return strMICR;
}

// Read magnetic stripe
CString CAPIControl::ReadStripe(EPBS_UINT32 dwFormat, EPBS_UINT32 dwDataType, bool bSave)
{
	int nResult = SUCCESS;
	EPBS_UINT32 dwReadLength = 0;
	EPBS_UINT32 dwDetail = 0;
	EPBS_PUINT8 lpbRead = NULL;
	CString strMSR;
	strMSR.Empty();
	CString strSave;
	strSave.Empty();

	// Lock the port
	CheckResponse(BiLockPort(m_nHandle, 0));
	if(m_ErrFlag)
	{
		return strMSR;
	}

	// Set magnetic stripe read settings in driver
	ConfigureMSRW(dwFormat, dwDataType);

	// Set MF_MSRW structure in driver
	nResult = BiMSRWSetting(m_nHandle, &m_Msrw);
	if(nResult == SUCCESS)
	{
		// Process magnetic stripe reading
		nResult = BiMSRWReadStripeData(m_nHandle, PASSBOOK_INSERT_TIMEOUT);
		if(nResult == SUCCESS)
		{

			// Retrieve data size for magnetic stripe
			nResult = BiMSRWGetStripeData(m_nHandle, 1, NULL, 0, &dwReadLength, &dwDetail);

			if(dwReadLength != 0)
			{
				lpbRead = new(std::nothrow) EPBS_UINT8[dwReadLength];
				memset(lpbRead, 0, (sizeof(EPBS_UINT8) * dwReadLength));
			}

			// Retrieve data for magnetic stripe
			nResult = BiMSRWGetStripeData(m_nHandle, 1, lpbRead, dwReadLength, &dwReadLength, &dwDetail);
			if(dwReadLength != 0)
			{
				strMSR = lpbRead;
				strSave = lpbRead;
			}
			else
			{
				if(nResult == SUCCESS)
				{
					strMSR = _T(" < No Data > ");
				}
			}
		}

		// Eject passbook
		CheckResponse(BiEjectPaper(m_nHandle, MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT));
	}

	if(lpbRead != NULL)
	{
		delete[] lpbRead;
		lpbRead = NULL;
	}

	// Unlock the port
	CheckResponse(BiUnlockPort(m_nHandle));

	if(nResult != SUCCESS)
	{
		strMSR += _T(" ( ");
		strMSR += GetResultString(nResult);
		strMSR += _T(" ) ");
	}

	if(bSave)
	{
		SaveMSR(nResult, dwDetail, strSave);
	}

	return strMSR;
}

// Write magnetic stripe
void CAPIControl::WriteStripe(EPBS_UINT32 dwFormat, EPBS_UINT32 dwDataType, LPCSTR lpWriteString)
{
	// Lock the port
	CheckResponse(BiLockPort(m_nHandle, 0));
	if(m_ErrFlag)
	{
		return;
	}

	// Set magnetic stripe write settings in driver
	ConfigureMSRW(dwFormat, dwDataType);

	// Set MF_MSRW structure in driver
	CheckResponse(BiMSRWSetting(m_nHandle, &m_Msrw));
	if(m_ErrFlag)
	{
		// Unlock the port
		CheckResponse(BiUnlockPort(m_nHandle));
		return;
	}

	// Process magnetic stripe writing
	CheckResponse(BiMSRWWriteStripeData(m_nHandle, PASSBOOK_INSERT_TIMEOUT, (EPBS_PUINT8)lpWriteString, (EPBS_UINT32)strlen(lpWriteString)));

	// Eject passbook
	CheckResponse(BiEjectPaper(m_nHandle, MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT));

	// Unlock the port
	CheckResponse(BiUnlockPort(m_nHandle));
}

// Erase magnetic stripe
void CAPIControl::ClearStripe(EPBS_UINT32 dwFormat, EPBS_UINT32 dwDataType)
{
	// Lock the port
	CheckResponse(BiLockPort(m_nHandle, 0));
	if(m_ErrFlag)
	{
		return;
	}

	// Set magnetic stripe erase settings in driver
	ConfigureMSRW(dwFormat, dwDataType);

	// Set MF_MSRW structure in driver
	CheckResponse(BiMSRWSetting(m_nHandle, &m_Msrw));
	if(m_ErrFlag)
	{
		// Unlock the port
		CheckResponse(BiUnlockPort(m_nHandle));
		return;
	}

	// Process magnetic stripe erasing
	CheckResponse(BiMSRWClearStripeData(m_nHandle, PASSBOOK_INSERT_TIMEOUT));

	// Eject passbook
	CheckResponse(BiEjectPaper(m_nHandle, MF_EJECT_FRONT, PAPER_EJECT_TIMEOUT));

	// Unlock the port
	CheckResponse(BiUnlockPort(m_nHandle));
}

// Setting MICR settings
void CAPIControl::ConfigureMICR(EPBS_UINT8 bSelect, EPBS_UINT8 bFont)
{
	// MF_BASE01
	m_tBase01.iSize = sizeof(MF_BASE01);
	m_tBase01.iVersion = MF_BASE_VERSION01;

	// Retrieve default values for MF_BASE01 structure
	CheckResponse(BiSCNMICRFunction(m_nHandle, &m_tBase01, MF_GET_BASE_DEFAULT));
	if(m_ErrFlag)
	{
		return;
	}

	m_tBase01.dwTimeout = PAPER_INSERT_TIMEOUT;

	// Set structure MF_BASE01 in driver
	CheckResponse(BiSCNMICRFunction(m_nHandle, &m_tBase01, MF_SET_BASE_PARAM));
	if(m_ErrFlag)
	{
		return;
	}


	// MF_MICR
	m_tMicr.iSize = sizeof(MF_MICR);
	m_tMicr.iVersion = MF_MICR_VERSION;

	// Retrieve default values for MF_MICR structure
	CheckResponse(BiSCNMICRFunction(m_nHandle, &m_tMicr, MF_GET_MICR_DEFAULT));
	if(m_ErrFlag)
	{
		return;
	}

	m_tMicr.bFont = bFont;
	m_tMicr.bMicOcrSelect = bSelect;

	// Set MF_MICR structure in driver
	CheckResponse(BiSCNMICRFunction(m_nHandle, &m_tMicr, MF_SET_MICR_PARAM));
	if(m_ErrFlag)
	{
		return;
	}
}

// Settings for magnetic stripe reading, writing, and erasing
void CAPIControl::ConfigureMSRW(EPBS_UINT32 dwFormat, EPBS_UINT32 dwDataType)
{
	// MF_MSRW
	m_Msrw.iSize = sizeof(MF_MSRW);
	m_Msrw.iVersion = MF_MSRW_VERSION;
	m_Msrw.dwRecordFormat = dwFormat;
	m_Msrw.dwIBMDataType = dwDataType;
	m_Msrw.dwReadRetry = 0;
	m_Msrw.dwWriteAndClearRetry = 0;
	m_Msrw.dwBlockCount = 1;
	m_Msrw.dwVerticalAdjust = 0;
}

// Saving MICR data to file
void CAPIControl::SaveMICR(int nResult)
{
	CString strOutput;
	strOutput.Empty();
	CString strTemp;
	strTemp.Empty();

	strOutput += _T("# MICR Data Output\n\n");
	strOutput += _T("# Magnetic Character Results\n");
	strTemp.Format(_T("Result = %s\n"), GetResultString(m_tMicr.iRet));
	strOutput += strTemp;
	strTemp.Format(_T("Status = %d\n"), m_tMicr.bStatus);
	strOutput += strTemp;
	strTemp.Format(_T("Detail = %d\n\n"), m_tMicr.bDetail);
	strOutput += strTemp;
	strTemp.Format(_T("MicrResult = %s\n"), GetResultString(nResult));
	strOutput += strTemp;
	strTemp.Format(_T("MicrString = %s\n"), m_tMicr.szMicrStr);
	strOutput += strTemp;

	FILE* pFile = _tfopen(_T("Micr.txt"), _T("wt"));
	if(pFile == NULL)
	{
		return;
	}

	fprintf(pFile, "%s", (const char *)strOutput);

	fclose(pFile);
}

// Saving magnetic stripe data to file
void CAPIControl::SaveMSR(int nResult, EPBS_UINT32 dwDetail, CString strStripeData)
{
	CString strOutput;
	strOutput.Empty();
	CString strTemp;
	strTemp.Empty();

	strOutput += _T("# MSR Data Output\n\n");
	strOutput += _T("# Magnetic Stripe Results\n");
	strTemp.Format(_T("RecordFormat = %s\n"), GetRecordFormatString(m_Msrw.dwRecordFormat));
	strOutput += strTemp;
	strTemp.Format(_T("IBMDataType = %s\n"), GetIBMDataTypeString(m_Msrw.dwIBMDataType));
	strOutput += strTemp;
	strTemp.Format(_T("Detail = %d\n\n"), dwDetail);
	strOutput += strTemp;
	strTemp.Format(_T("MSRResult = %s\n"), GetResultString(nResult));
	strOutput += strTemp;
	strTemp.Format(_T("MSRData = %s\n"), strStripeData);
	strOutput += strTemp;

	FILE* pFile = _tfopen(_T("MSR.txt"), _T("wt"));
	if(pFile == NULL)
	{
		return;
	}

	fprintf(pFile, "%s", (const char *)strOutput);

	fclose(pFile);
}

// Get device status
void CAPIControl::GetStatus(EPBS_UINT32 dwStatusLength, EPBS_PUINT32 lpdwStatuses)
{
	CheckResponse(BiGetMultiStatus(m_nHandle, dwStatusLength, lpdwStatuses));
}

// Get error message strings
CString CAPIControl::GetResultString(int nResultCode)
{
	CString strResult;
	strResult.Empty();

	switch(nResultCode)
	{
		case SUCCESS:
			strResult = _T("SUCCESS");
			break;
		case ERR_UNKNOWN:
			strResult = _T("ERR_UNKNOWN");
			break;
		case ERR_TYPE:
			strResult = _T("ERR_TYPE");
			break;
		case ERR_OPENED:
			strResult = _T("ERR_OPENED");
			break;
		case ERR_NO_PRINTER:
			strResult = _T("ERR_NO_PRINTER");
			break;
		case ERR_NO_MEMORY:
			strResult = _T("ERR_NO_MEMORY");
			break;
		case ERR_HANDLE:
			strResult = _T("ERR_HANDLE");
			break;
		case ERR_TIMEOUT:
			strResult = _T("ERR_TIMEOUT");
			break;
		case ERR_ACCESS:
			strResult = _T("ERR_ACCESS");
			break;
		case ERR_PARAM:
			strResult = _T("ERR_PARAM");
			break;
		case ERR_NOT_SUPPORT:
			strResult = _T("ERR_NOT_SUPPORT");
			break;
		case ERR_BUFFER_OVER_FLOW:
			strResult = _T("ERR_BUFFER_OVER_FLOW");
			break;
		case ERR_NOT_FOUND:
			strResult = _T("ERR_NOT_FOUND");
			break;
		case ERR_MICR:
			strResult = _T("ERR_MICR");
			break;
		case ERR_NOT_EXEC:
			strResult = _T("ERR_NOT_EXEC");
			break;
		case ERR_MICR_NODATA:
			strResult = _T("ERR_MICR_NODATA");
			break;
		case ERR_MICR_BADDATA:
			strResult = _T("ERR_MICR_BADDATA");
			break;
		case ERR_MICR_NOISE:
			strResult = _T("ERR_MICR_NOISE");
			break;
		case ERR_EXEC_FUNCTION:
			strResult = _T("ERR_EXEC_FUNCTION");
			break;
		case ERR_OFFLINE:
			strResult = _T("ERR_OFFLINE");
			break;
		case ERR_PAPERINSERT_TIMEOUT:
			strResult = _T("ERR_PAPERINSERT_TIMEOUT");
			break;
		case ERR_PAPER_JAM:
			strResult = _T("ERR_PAPER_JAM");
			break;
		case ERR_PAPER_INSERT:
			strResult = _T("ERR_PAPER_INSERT");
			break;
		case ERR_UNLOCKED:
			strResult = _T("ERR_UNLOCKED");
			break;
		case ERR_NET_CONNECTED:
			strResult = _T("ERR_NET_CONNECTED");
			break;
		case ERR_MSRW_NODATA:
			strResult = _T("ERR_MSRW_NODATA");
			break;
		case ERR_MSRW_WRITE:
			strResult = _T("ERR_MSRW_WRITE");
			break;
		default:
			strResult.Format(_T("%d"), nResultCode);
			break;
	}

	return strResult;
}

// Get format string
CString CAPIControl::GetRecordFormatString(EPBS_UINT32 dwRecordFormat)
{
	CString strFormat;
	strFormat.Empty();

	switch(dwRecordFormat)
	{
		case EPS_BI_MSRW_FORMAT_IBM3604:
			strFormat = _T("IBM3604");
			break;
		case EPS_BI_MSRW_FORMAT_DINISO:
			strFormat = _T("DINISO");
			break;
		case EPS_BI_MSRW_FORMAT_ANSI:
			strFormat = _T("ANSI");
			break;
		case EPS_BI_MSRW_FORMAT_IBM4746:
			strFormat = _T("IBM4746");
			break;
		case EPS_BI_MSRW_FORMAT_ISO7811:
			strFormat = _T("ISO7811");
			break;
		case EPS_BI_MSRW_FORMAT_HT2751CIZ:
			strFormat = _T("HT2751CIZ");
			break;
		case EPS_BI_MSRW_FORMAT_ISO8484:
			strFormat = _T("ISO8484");
			break;
		default:
			strFormat.Format(_T("%d"), dwRecordFormat);
			break;
	}

	return strFormat;
}

// Get IBM Data Type string
CString CAPIControl::GetIBMDataTypeString(EPBS_UINT32 dwIBMDataType)
{
	CString strDataType;
	strDataType.Empty();

	switch(dwIBMDataType)
	{
		case EPS_BI_MSRW_EOM_NONE:
			strDataType = _T("NONE");
			break;
		case EPS_BI_MSRW_EOM_3C:
			strDataType = _T("3C");
			break;
		case EPS_BI_MSRW_EOM_3F:
			strDataType = _T("3F");
			break;
		default:
			strDataType.Format(_T("%d"), dwIBMDataType);
			break;
	}

	return strDataType;
}
