#ifndef _DATAMANAGER_H_
#define	_DATAMANAGER_H_

#include <vector>
#include "twain.h"
#include "EpcsTwain.h"

//============================================================================
//!	Structure of Barcode Priorities.
/*!
 */
//============================================================================
struct BarcodePrioritiesT
{
	unsigned short		count;
	TW_UINT16	Priorities[256];
};

//============================================================================
//!	Structure of MICR data.
/*!
 */
//============================================================================
struct MICRDataT
{
	TW_UINT16		magType;		///!< TWEI_MAGTYPE
	// TWMD_MICR
	TW_STR255		magData;		///!< TWEI_MAGDATA
};
typedef MICRDataT					EPMICR_DATA, *PEPMICR_DATA;
typedef std::vector<EPMICR_DATA>	EPMICRDataList;

//============================================================================
//!	Structure of OCR data.
/*!
 */
//============================================================================
struct OCRDataT
{
	TW_STR255		Data;			//!< TWEI_EPCS_OCRABDATA
};
typedef	OCRDataT					EPOCR_DATA, *PEPOCR_DATA;
typedef std::vector<EPOCR_DATA>		EPOCRDataList;

//============================================================================
//!	Structure of Barcode data.
/*!
 */
//============================================================================
struct BarcodeDataT
{
	TW_UINT32		Rotation;		///!< TWEI_BARCODEROTATION
	// TWBCOR_ROT0
	// TWBCOR_ROT90
	// TWBCOR_ROT180
	// TWBCOR_ROT270
	TW_UINT32		Type;			///!< TWEI_BARCODETYPE
	// TWBT_3OF9
	// TWBT_2OF5INTERLEAVED
	// TWBT_CODE128
	// TWBT_CODABAR
	// TWBT_UPCA
	// TWBT_UPCE
	// TWBT_EAN8
	// TWBT_EAN13
	TW_UINT32		BarcodeX;		///!< TWEI_BARCODEX
	// >= 0
	TW_UINT32		BarcodeY;		///!< TWEI_BARCODEY
	// >= 0

	TW_UINT32		TextLength;		///!< TWEI_BARCODETEXTLENGTH
	TW_STR255		Text;			///!< TWEI_BARCODETEXT
};
typedef BarcodeDataT				EPBARCODE_DATA, *PEPBARCODE_DATA;
typedef std::vector<EPBARCODE_DATA>	EPBarcodeDataList;

#define	RECOGDATA_MICR		0
#define RECOGDATA_OCR		1
#define RECOGDATA_BARCODE	2

typedef enum RECOG_STATE
{
	RECOG_DISABLE	= 0,
	RECOG_ENABLE	= 1,
	RECOG_DECORD	= 2,
};

class DataManager
{
public:
	DataManager(void);
	~DataManager(void);

	RECOG_STATE			GetDecodeMicr(EPMICR_DATA *micr);
	RECOG_STATE			GetDecodeOcr(EPOCR_DATA *ocr);
	RECOG_STATE			GetDecodeBarcode(EPBarcodeDataList *barcode);

	void				SetBitmapHandle(BITMAPINFOHEADER *pBIH);
	void				SetDecodeMicrData(EPMICR_DATA *pData, size_t size);
	void				SetDecodeOcrData(EPOCR_DATA *pData, size_t size);
	void				SetDecodeBarcodeData(EPBarcodeDataList *pData);
	HBITMAP				GetBitmapHandle(void);
	void				FreeBitmapHandle(void);
	void				ResetRecogData(void);
	void				SetRecogDataState(TW_UINT32 recogType, RECOG_STATE state);

private:
	HBITMAP				m_hBitmap;
	RECOG_STATE			m_MicrState;
	RECOG_STATE			m_OcrState;
	RECOG_STATE			m_BarcodeState;
	EPMICR_DATA			m_dataMicr;
	EPOCR_DATA			m_dataOcr;
	EPBarcodeDataList	m_dataBarcode;

	void				ResetMicrData(void);
	void				ResetOcrData(void);
	void				ResetBarcodeData(void);
};
#endif // _DATAMANAGER_H_