#ifndef __TWAININTERFACE_H__
#define __TWAININTERFACE_H__

#include "DataManager.h"

struct MemoryFunctions
{
	DSM_MEMALLOCATE		fnMemAlloc;			//!< Entry point of allocating memory function.
	DSM_MEMFREE	    	fnMemFree;			//!< Entry point of freeing memory function.
	DSM_MEMLOCK      	fnMemLock;			//!< Entry point of locking memory function.
	DSM_MEMUNLOCK      	fnMemUnlock;		//!< Entry point of unlocking memory function.
};

#define MAX_RETURNCODE						10
#define	MAX_CONDITIONCODE					24
#define MAX_DG								3
#define MAX_DAT								29
#define MAX_MSG								35

#define STR_FORMAT_RETURNCODE				_T("RC	= %s")
#define STR_FORMAT_CONDITIONCODE			_T("CC	= %s")
#define STR_FORMAT_DAT						_T("DAT	= %s")
#define STR_FORMAT_DG						_T("DG	= %s")
#define STR_FORMAT_MSG						_T("MSG	= %s")
#define STR_FORMAT_RETURNCODE_UNKNOWN		_T("RC	= %#x")
#define STR_FORMAT_CONDITIONCODE_UNKNOWN	_T("CC	= %#x")
#define STR_FORMAT_DAT_UNKNOWN				_T("DAT	= %#x")
#define STR_FORMAT_DG_UNKNOWN				_T("DG	= %#x")
#define STR_FORMAT_MSG_UNKNOWN				_T("MSG	= %#x")

typedef enum STATECODE
{
	STATE_CLOSE_DSM		= 0,
	STATE_OPEN_DSM		= 1,
	STATE_LOAD_DS		= 2,
	STATE_ENABLE_DS		= 3,
};

struct TblMessage
{
	const TCHAR	*tcsItemName;
	TW_UINT16		ItemId;
};

class TwainInterface
{
public:
	TwainInterface();
	virtual ~TwainInterface();

	TW_UINT16		DSM_Entry		(TW_UINT32 _DG,TW_UINT16 _DAT, TW_UINT16 _MSG, TW_MEMREF _pData, BOOL bShowStatus = TRUE);
	TW_HANDLE		DSM_Alloc		(TW_UINT32 _size);
	void			DSM_Free		(TW_HANDLE _handle);
	TW_MEMREF		DSM_Lock		(TW_HANDLE _handle);
	void			DSM_Unlock		(TW_HANDLE _handle);

	BOOL			connectDSM		(HWND hParent);
	BOOL			selectDS		(HWND hParen,  pTW_IDENTITY pDsIdentity);
	BOOL			loadDS			(HWND hParent);
	BOOL			Acquire			(HWND hParent, TW_BOOL bShowUI);
	BOOL			disableDS		(HWND hParent);
	BOOL			unloadDS		(HWND hParent);
	BOOL			disconnectDSM	(HWND hParent);
	BOOL			IsEnabledDS		(void)
					{
						return (m_status == STATE_ENABLE_DS);
					}
	TW_IDENTITY		GetTWIdentity(void)
					{
						return m_DataSource;
					}
	STATECODE		GetStateCode(void);

private:
	void			fillIdentity(TW_IDENTITY& _identity);
	void			GetTwainStatus(HWND hParent, TW_UINT32 _DG,TW_UINT16 _DAT, TW_UINT16 _MSG, TW_UINT16 _twrc);

	TW_IDENTITY		m_MyInfo;
	TW_IDENTITY		m_DataSource;
	TW_ENTRYPOINT	m_DSMEntry;
	HMODULE			m_hDSM;
	STATECODE		m_status;

};

#endif // __TWAININTERFACE_H__
